<?php

namespace Drupal\Tests\permission_group\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\permission_group\Entity\PermissionGroup;

/**
 * Tests the Permission Group module doesn't cause unlimited recursion.
 *
 * @group permission_group
 */
class PermissionGroupRecursionTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['user', 'permission_group'];

  /**
   * {@inheritdoc}
   */
  protected function setUp() : void {
    parent::setUp();
    $this->installEntitySchema('user');
  }

  /**
   * Tests recursive groups.
   */
  public function testRecursion() {
    /** @var \Drupal\permission_group\Entity\PermissionGroup $group_a */
    /** @var \Drupal\permission_group\Entity\PermissionGroup $group_b */
    /** @var \Drupal\permission_group\Entity\PermissionGroup $group_c */
    $group_a = PermissionGroup::create(['id' => 'a']);
    $group_a->setPermissions(['perm1', 'perm2']);
    $group_a->save();

    $group_b = PermissionGroup::create(['id' => 'b']);
    $group_b->setPermissions(['perm1', 'perm3']);
    $group_b->setPermissionGroups(['a']);
    $group_b->save();

    $group_a->setPermissionGroups(['b'])->save();

    $this->assertSame(['perm1', 'perm2'], $group_a->permissions());
    $this->assertSame(['perm1', 'perm2', 'perm3'], $group_a->permissions(TRUE));

    // A longer chain of groups.
    $group_c = PermissionGroup::create(['id' => 'c']);
    $group_c->setPermissions(['perm2', 'perm4']);
    $group_c->setPermissionGroups(['a']);
    $group_c->save();

    $this->assertSame(['perm1', 'perm2', 'perm3', 'perm4'], $group_c->permissions(TRUE));
    $this->assertFalse($group_a->restrictAccess());
    $this->assertFalse($group_b->restrictAccess());
    $this->assertFalse($group_c->restrictAccess());

    // Add a restricted access permission.
    $group_a->setPermissions(['perm1', 'perm2', 'administer permissions']);
    $group_a->save();
    $this->assertSame(['administer permissions', 'perm1', 'perm2', 'perm3', 'perm4'], $group_c->permissions(TRUE));
    $this->assertTrue($group_a->restrictAccess());
    $this->assertTrue($group_b->restrictAccess());
    $this->assertTrue($group_c->restrictAccess());

    // Break the loop.
    $group_b->setPermissionGroups([]);
    $group_b->save();
    $this->assertTrue($group_a->restrictAccess());
    $this->assertFalse($group_b->restrictAccess());
    $this->assertTrue($group_c->restrictAccess());
  }

}
